
#include <config.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <regex.h>
#include <glib/gi18n.h>
#include <glib.h>
#include "tsc-ica-provider.h"
#include "tsc-ica-connection.h"
#include "tsc-util.h"

#define APPSRV_GROUP_NAME "ApplicationServers"

static TSCConnection *
tsc_ica_provider_create_connection (TSCProvider *provider)
{
	return TSC_CONNECTION (g_object_new (TSC_TYPE_ICA_CONNECTION, NULL));
}

static TSCEditDialog *
tsc_ica_provider_create_edit_dialog (TSCProvider *provider,
					 TSCConnection *connection)
{
	char *argv[] = { WFCMGR_PATH , NULL};
	GError *error = NULL;

	if (!g_file_test (WFCMGR_PATH, G_FILE_TEST_EXISTS)) {
		tsc_util_show_error ("Error", "Can't find wfcmgr");
		return NULL;
	}

	if (!g_spawn_async (NULL, argv, NULL, 0, NULL, NULL, NULL, &error)) {
		if (error) {
			tsc_util_show_error ("Error", error->message);
			g_error_free (error);
		} else {
			tsc_util_show_error ("Error", "Failed to start wfcmgr");
		}
	}

	return NULL;
}

static void
tsc_ica_provider_class_init (TSCIcaProviderClass *klass)
{
	TSC_PROVIDER_CLASS (klass)->create_connection = tsc_ica_provider_create_connection;
	TSC_PROVIDER_CLASS (klass)->create_edit_dialog = tsc_ica_provider_create_edit_dialog;
}

static void
tsc_ica_provider_init (TSCIcaProvider *provider)
{
	TSCProvider *parent = TSC_PROVIDER (provider);

	parent->name = "ICA";
	parent->display_name = _("Citrix ICA Connection");
	parent->description = _("Create a connection which uses the Citrix ICA client");
	parent->icon_name = "gnome-fs-network";
	parent->enabled = tsc_util_program_exists (WFICA_PATH);
}

GType
tsc_ica_provider_get_type (void)
{
	static GType type = 0;

	if (!type) {
		static GTypeInfo type_info = {
			sizeof (TSCIcaProviderClass),
			NULL, NULL,
			(GClassInitFunc) tsc_ica_provider_class_init,
			NULL, NULL,
			sizeof (TSCIcaProvider),
			0,
			(GInstanceInitFunc) tsc_ica_provider_init
		};
		
		type = g_type_register_static (TSC_TYPE_PROVIDER,
					       "TSCIcaProvider",
					       &type_info, 0);
	}
	
	return type;

}

TSCIcaProvider *
tsc_ica_provider_new (void)
{
	return g_object_new (TSC_TYPE_ICA_PROVIDER, NULL);
}

static GKeyFile *
open_appsrv (void)
{
	GKeyFile *key_file;
	char *appsrv_path, *data;
	gsize data_len;
	int i;

	data = NULL;
	key_file = NULL;
	
	appsrv_path = g_build_filename (g_get_home_dir (), ".ICAClient", "appsrv.ini", NULL);
	if (!g_file_test (appsrv_path, G_FILE_TEST_IS_REGULAR)) {
		goto cleanup;
	}

	if (!g_file_get_contents (appsrv_path, &data, &data_len, NULL)) {
		goto cleanup;
	}

	/* replace the ';' comments with '#' so GKeyFile can cope */
	for (i = 0; i < data_len; i++) {
		if (data[i] == ';' && (i == 0 || data[i - 1] == '\n')) {
			data[i] = '#';
		}
	}

	key_file = g_key_file_new ();
	if (!g_key_file_load_from_data (key_file, data, data_len,
					G_KEY_FILE_NONE, NULL)) {
		g_key_file_free (key_file);
		key_file = NULL;
	}

cleanup:
	g_free (appsrv_path);

	if (data) {
		g_free (data);
	}

	return key_file;
}

static GList *
find_app_descriptions (void)
{
	GKeyFile *key_file;
	GList *list;
	char **keys;
	int i;

	list = NULL;

	key_file = open_appsrv ();
	if (!key_file) {
		return list;
	}

	keys = g_key_file_get_keys (key_file, APPSRV_GROUP_NAME,
				    NULL, NULL);
	if (!keys) {
		g_key_file_free (key_file);
		return list;
	}

	for (i = 0; keys[i]; i++) {
		list = g_list_prepend (list, g_strdup (keys[i]));
	}

	g_key_file_free (key_file);
	return list;
}

void
tsc_ica_provider_load_connections (TSCIcaProvider *provider, TSCManager *manager)
{
	GList *apps, *l;
	TSCIcaConnection *connection;
	
	apps = find_app_descriptions ();
	if (!apps) {
		return;
	}

	for (l = apps; l; l = l->next) {
		connection = tsc_ica_connection_new ((const char *) l->data);
		TSC_CONNECTION (connection)->provider = TSC_PROVIDER (provider);
		
		tsc_manager_add_connection (manager, TSC_CONNECTION (connection));
	}

	g_list_foreach (apps, (GFunc) g_free, NULL);
	g_list_free (apps);
}
